@@ -39,12 +39,13 @@ sub run {
 						 '.deleted' => undef,
 						 '.conflict' => undef,
 					       });
-
     $self->{rev} = $target->{repos}->fs->youngest_rev unless defined $self->{rev};
     $target->{report} = $report;
     $target->{copath} = $copath;
 
-    return $self->SUPER::run ($target);
+    return $self->SUPER::run (SVK::Target->new (%$target,
+						report => $report,
+						copath => $copath));
 }
 
 1;
